package com.yaoxiang.planning.action

import com.yaoxiang.planning.domain.AnnualPlan
import com.yaoxiang.planning.model.Reply
import com.yaoxiang.planning.service.AnnualPlanService
import io.swagger.annotations.Api
import io.swagger.annotations.ApiImplicitParam
import io.swagger.annotations.ApiImplicitParams
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*
import org.thymeleaf.util.DateUtils
import java.util.*

@Api(tags = ["年度计划"])
@RestController
@RequestMapping("/rest/planning/annual")
class AnnualAction {

    @Autowired
    private lateinit var annualPlanService: AnnualPlanService

    @ApiOperation("添加年度计划")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "year", value = "年份", paramType = "query")
    )
    @PostMapping("add")
    fun add(name: String, content: String, year: Int): Reply<Any> {
        val result = annualPlanService.add(name, content, year)
        return if (result) Reply.ok() else Reply.fail("当年年度计划已存在")
    }

    @ApiOperation("更新年度计划")
    @ApiImplicitParams(
        ApiImplicitParam(name = "id", value = "年度计划id", paramType = "query"),
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "year", value = "年份", paramType = "query"),
        ApiImplicitParam(name = "status", value = "状态  0准备 1执行 2暂搁 3完成 4中止", paramType = "query"),
    )
    @PostMapping("update")
    fun update(id: Long, name: String, content: String, year: Int, status: Int): Reply<Any> {
        val result = annualPlanService.update(id, name, content, year, status)
        return Reply(result)
    }

    @ApiOperation("删除年度计划")
    @PostMapping("delete")
    fun delete(id: Long): Reply<Any> {
        val result = annualPlanService.delete(id)
        return Reply(result)
    }

    @ApiOperation("获取年度计划")
    @GetMapping("current")
    fun current(@RequestParam(required = false) year: Int? = null): Reply<AnnualPlan> {
        val y: Int = year ?: DateUtils.year(Date())
        val optional = annualPlanService.findByYear(y)
        return Reply(optional.isPresent, "", optional.orElse(null))
    }

    @ApiOperation("获取年度计划列表")
    @GetMapping("list")
    fun list(): Reply<List<AnnualPlan>> {
        val list = annualPlanService.list()
        return Reply.ok(list)
    }

    @ApiOperation("评价")
    @PostMapping("evaluate")
    fun evaluate(id: Long, evaluation: String): Reply<Any> {
        val result = annualPlanService.evaluate(id, evaluation)
        return Reply(result)
    }
}