package com.yaoxiang.planning.action

import com.yaoxiang.planning.domain.PlanningItem
import com.yaoxiang.planning.model.Reply
import com.yaoxiang.planning.service.PlanningItemService
import io.swagger.annotations.Api
import io.swagger.annotations.ApiImplicitParam
import io.swagger.annotations.ApiImplicitParams
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*

@Api(tags = ["分计划"])
@RestController
@RequestMapping("/rest/planning/planningItem")
class PlanningItemAction {

    @Autowired
    private lateinit var planningItemService: PlanningItemService

    @PostMapping("addItem")
    @ApiOperation("增加分计划")
    fun addItem(@RequestBody item: PlanningItem): Reply<Any> {
        val result = planningItemService.addItem(item)
        return Reply(result)
    }

    @ApiOperation("批量增加计划项")
    @PostMapping("addItems")
    fun addItems(@RequestBody items: List<PlanningItem>): Reply<Any> {
        val result = planningItemService.addItems(items)
        return Reply(result)
    }

    @ApiOperation("更新子计划项")
    @PostMapping("update")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "内容", paramType = "query"),
        ApiImplicitParam(name = "status", value = "状态  0准备 1执行 2暂搁 3完成 4中止", paramType = "query"),
        ApiImplicitParam(name = "remark", value = "备注", paramType = "query"),
        ApiImplicitParam(name = "departmentId", value = "部门Id", paramType = "query")
    )
    fun update(
        id: Long, name: String, content: String, status: Int,
        @RequestParam(required = false) remark: String?,
        @RequestParam(required = false) departmentId: Long?
    ): Reply<Any> {
        val result = planningItemService.update(id, name, content, status, remark, departmentId)
        return Reply(result)
    }

    @ApiOperation("评价子计划")
    @PostMapping("evaluate")
    fun evaluate(id: Long, evaluation: String): Reply<Any> {
        val result = planningItemService.evaluate(id, evaluation)
        return Reply(result)
    }

    @ApiOperation("查找给定年度计划或者季度计划的所有分计划")
    @PostMapping("findByPlanningIdAndType")
    fun findByPlanningIdAndType(planningId: Long, type: Int): Reply<List<PlanningItem>> {
        val result = planningItemService.findByPlanningIdAndType(planningId, type)
        return Reply.ok(result)
    }

    @ApiOperation("删除分计划")
    @PostMapping("deleteItem")
    fun deleteItem(id: Long): Reply<Any> {
        val result = planningItemService.deleteItem(id)
        return Reply(result)
    }

    @ApiOperation("批量删除分计划")
    @PostMapping("deleteItems")
    fun deleteItems(@RequestBody ids: List<Long>): Reply<Any> {
        val result = planningItemService.deleteItems(ids)
        return Reply(result)
    }

    @ApiOperation("删除给定年度计划或者季度计划的所有分计划")
    @PostMapping("deleteByPlanningIdAndType")
    @ApiImplicitParams(
        ApiImplicitParam(name = "planningId", value = "计划Id", paramType = "query"),
        ApiImplicitParam(name = "type", value = "计划类型", paramType = "query")
    )
    fun deleteItem(planningId: Long, type: Int): Reply<Any> {
        val result = planningItemService.deleteByPlanningIdAndType(planningId, type)
        return Reply(result)
    }

}