package com.yaoxiang.planning.action

import com.yaoxiang.planning.domain.QuarterlyPlan
import com.yaoxiang.planning.model.Reply
import com.yaoxiang.planning.service.QuarterlyPlanService
import io.swagger.annotations.Api
import io.swagger.annotations.ApiImplicitParam
import io.swagger.annotations.ApiImplicitParams
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*
import org.thymeleaf.util.DateUtils
import java.util.*

@Api(tags = ["季度计划"])
@RestController
@RequestMapping("/rest/planning/quarterly")
class QuarterlyAction {

    @Autowired
    private lateinit var quarterlyPlanService: QuarterlyPlanService

    @ApiOperation("添加季度计划")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "year", value = "年份", paramType = "query"),
        ApiImplicitParam(name = "quarter", value = "季度", paramType = "query"),
        ApiImplicitParam(name = "departmentId", value = "部门Id", paramType = "query")
    )
    @PostMapping("add")
    fun add(
        name: String, content: String, year: Int, quarter: Int,
        @RequestParam(required = false) departmentId: Long?
    ): Reply<Any> {
        val result = quarterlyPlanService.add(name, content, year, quarter, departmentId)
        return Reply(result)
    }

    @ApiOperation("更新季度计划")
    @PostMapping("update")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "year", value = "年份", paramType = "query"),
        ApiImplicitParam(name = "quarter", value = "季度", paramType = "query"),
        ApiImplicitParam(name = "statue",value = "状态 0准备 1执行 2暂搁 3完成 4中止",paramType = "query"),
        ApiImplicitParam(name = "departmentId", value = "部门Id", paramType = "query")
    )
    fun update(
        id: Long, name: String, content: String, year: Int, quarter: Int, status: Int,
        @RequestParam(required = false) departmentId: Long?
    ): Reply<Any> {
        val result = quarterlyPlanService.update(id, name, content, year, quarter, status, departmentId)
        return Reply(result)
    }

    @ApiOperation("删除季度计划")
    @PostMapping("delete")
    fun delete(id: Long): Reply<Any> {
        val result = quarterlyPlanService.delete(id);
        return Reply(result);
    }

    @ApiOperation("获取季度计划")
    @ApiImplicitParam(name = "year", value = "year", paramType = "query")
    @GetMapping("list")
    fun list(
        @RequestParam(required = false) year: Int? = null,
        @RequestParam(required = false) departmentId: Long?
    ): Reply<List<QuarterlyPlan>> {
        val y: Int = year ?: DateUtils.year(Date())
        val result = quarterlyPlanService.list(y, departmentId)
        return Reply.ok(result)
    }

    @ApiOperation("评价")
    @PostMapping("evaluate")
    fun evaluate(id: Long, evaluation: String): Reply<Any> {
        val result = quarterlyPlanService.evaluate(id, evaluation)
        return Reply(result)
    }
}