package com.yaoxiang.planning.action

import com.yaoxiang.planning.domain.UserInfo
import com.yaoxiang.planning.model.Reply
import com.yaoxiang.planning.service.UserService
import io.swagger.annotations.Api
import io.swagger.annotations.ApiImplicitParam
import io.swagger.annotations.ApiImplicitParams
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*

@Api(tags = ["用户"])
@RestController
@RequestMapping("/rest/planning/user")
class UserAction {

    @Autowired
    private lateinit var userService: UserService

    @ApiOperation("当前用户id")
    @GetMapping("curId")
    fun currentUserId(): Reply<Long?> {
        val result = userService.currentUserId()
        return Reply.ok(result)
    }

    @ApiOperation("新增用户")
    @PostMapping("add")
    fun add(
        username: String, name: String, password: String, phone: String,
        @RequestParam(required = false) departmentId: Long?
    ): Reply<Any> {
        val result = userService.add(username, name, password, phone, departmentId)
        return if (result) Reply.ok() else Reply.fail("新增失败")
    }

    @ApiOperation("更新用户信息")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "昵称", paramType = "query"),
        ApiImplicitParam(name = "age", value = "职责", paramType = "query"),
        ApiImplicitParam(name = "gender", value = "性别", paramType = "query"),
        ApiImplicitParam(name = "email", value = "上级部门Id", paramType = "query"),
        ApiImplicitParam(name = "phone", value = "手机号码", paramType = "query")
    )
    @PostMapping("updateInfo")
    fun updateInfo(
        id: Long, name: String, @RequestParam(required = false) age: Int?,
        @RequestParam(required = false) gender: Int?,
        @RequestParam(required = false) email: String?,
        @RequestParam(required = false) phone: String
    ): Reply<Any> {
        val result = userService.updateInfo(id, name, age, gender, email, phone)
        return if (result) Reply.ok() else Reply.fail("更新失败")
    }

    @ApiOperation("更新密码")
    @PostMapping("updatePassword")
    fun updatePassword(id: Long, oldPassword: String, newPassword: String): Reply<Any> {
        val result = userService.updatePassword(id, oldPassword, newPassword)
        return if (result) Reply.ok() else Reply.fail("更新密码失败")
    }

    @ApiOperation("重置密码")
    @PostMapping("resetPassword")
    fun resetPassword(id: Long, secret: String?): Reply<Any> {
        val result = userService.resetPassword(id, secret)
        return if (result) Reply.ok() else Reply.fail("重置密码失败")
    }

    @ApiOperation("当前用户")
    @GetMapping("cur")
    fun currentUser(): Reply<UserInfo> {
        val result = userService.currentUser()
        return Reply.ok(result)
    }

    @ApiOperation("更新用户部门")
    @PostMapping("updateDepartment")
    fun updateDepartment(id: Long, departmentId: Long): Reply<Any> {
        val result = userService.updateDepartment(id, departmentId)
        return if (result) Reply.ok() else Reply.fail("更新失败")
    }

    @ApiOperation("删除用户")
    @PostMapping("delete")
    fun delete(id: Long): Reply<Any> {
        val result = userService.delete(id)
        return if (result) Reply.ok() else Reply.fail("删除失败")
    }

    @ApiOperation("获取用户列表")
    @GetMapping("list")
    fun list(): Reply<List<UserInfo>> {
        val list = userService.list()
        return Reply.ok(list)
    }
}