package com.yaoxiang.planning.action

import com.yaoxiang.planning.domain.WeeklyPlan
import com.yaoxiang.planning.model.Reply
import com.yaoxiang.planning.service.WeeklyPlanService
import io.swagger.annotations.Api
import io.swagger.annotations.ApiImplicitParam
import io.swagger.annotations.ApiImplicitParams
import io.swagger.annotations.ApiOperation
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.*
import org.thymeleaf.util.DateUtils
import java.util.*

@Api(tags = ["周计划"])
@RestController
@RequestMapping("/rest/planning/weekly")
class WeeklyAction {

    @Autowired
    private lateinit var weeklyPlanService: WeeklyPlanService

    @ApiOperation("添加周计划")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "startTime", value = "计划时间", paramType = "query"),
        ApiImplicitParam(name = "year", value = "年份", paramType = "query")
    )
    @PostMapping("add")
    fun add(
        name: String, content: String,
        @RequestParam(required = false) userId: Long?,
        @RequestParam(required = false) startTime: Long? = null
    ): Reply<Any> {
        val start = startTime ?: System.currentTimeMillis()
        val year = DateUtils.year(Date(start))
        weeklyPlanService.add(name, content, userId, start, year)
        return Reply.ok()
    }

    @ApiOperation("更新周计划")
    @ApiImplicitParams(
        ApiImplicitParam(name = "name", value = "名称", paramType = "query"),
        ApiImplicitParam(name = "content", value = "计划内容", paramType = "query"),
        ApiImplicitParam(name = "startTime", value = "计划时间", paramType = "query"),
        ApiImplicitParam(name = "status", value = "状态 0准备 1执行 2暂搁 3完成 4中止", paramType = "query")
    )
    @PostMapping("update")
    fun update(id: Long, name: String, content: String, startTime: Long, status: Int): Reply<Any> {
        val year = DateUtils.year(Date(startTime))
        val result = weeklyPlanService.update(id, name, content, startTime, year, status)
        return Reply(result)
    }

    @ApiOperation("删除周计划")
    @PostMapping("delete")
    fun delete(id: Long): Reply<Any> {
        val result = weeklyPlanService.delete(id)
        return Reply(result)
    }

    @GetMapping("listCurrentMonth")
    @ApiOperation("获取个人当月的计划列表")
    fun listCurrentMonth(
        @RequestParam(required = false) startTime: Long?,
        @RequestParam(required = false) userId: Long?
    ): Reply<Map<String, List<WeeklyPlan>>> {
        val start = startTime ?: System.currentTimeMillis()
        val result = weeklyPlanService.mapCurrentMonth(start, userId)
        return Reply.ok(result)
    }

    @ApiOperation("评价")
    @PostMapping("evaluate")
    fun evaluate(id: Long, evaluation: String): Reply<Any> {
        val result = weeklyPlanService.evaluate(id, evaluation)
        return Reply(result)
    }

//    @ApiOperation("获取所有人的周计划列表")
//    @GetMapping("listAllOfCurrentWeek")
//    fun listAllOfCurrentWeek(@RequestParam(required = false) startTime: Long?): Reply<Map<String, List<WeeklyPlan>>> {
//        val start = startTime ?: System.currentTimeMillis()
//        val result = weeklyPlanService.mapCurrentWeek(start, false)
//        return Reply.ok(result)
//    }

}