package com.yaoxiang.planning.service

import com.yaoxiang.planning.domain.AnnualPlan
import com.yaoxiang.planning.model.PlanningType
import com.yaoxiang.planning.repository.AnnualPlanRepo
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.util.*

@Service
class AnnualPlanService {

    @Autowired
    private lateinit var annualPlanRepo: AnnualPlanRepo

    @Autowired
    private lateinit var planningService: PlanningService

    @Autowired
    private lateinit var planningItemService: PlanningItemService

    @Transactional
    fun add(name: String, content: String, year: Int): Boolean {
        if (exists(year)) {
            return false
        }
        val planning = AnnualPlan()
        planning.type = PlanningType.ANNUAL.ordinal
        planningService.add(name, content, year, planning)
        return true
    }

    @Transactional(readOnly = true)
    fun exists(year: Int): Boolean {
        val optional = annualPlanRepo.findFirstByYear(year)
        return optional.isPresent
    }

    fun getByYear(year: Int): Optional<AnnualPlan> {
        val optional = findByYear(year)
        if (optional.isPresent) {
            val plan = optional.get()
            val list = planningItemService.findByPlanningIdAndType(plan.id!!, plan.type!!)
            plan.items = list
        }
        return optional
    }

    @Transactional(readOnly = true)
    fun findByYear(year: Int): Optional<AnnualPlan> {
        return annualPlanRepo.findFirstByYear(year)
    }

    fun get(id: Long): Optional<AnnualPlan> {
        return annualPlanRepo.findById(id)
    }

    @Transactional
    fun update(id: Long, name: String, content: String, year: Int, status: Int): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        val plan = optional.get()
        plan.name = name
        plan.content = content
        plan.status = status
        plan.year = year
        annualPlanRepo.save(plan)
        return true
    }

    @Transactional
    fun delete(id: Long): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        annualPlanRepo.delete(optional.get())
        return true
    }

    @Transactional(readOnly = true)
    fun list(): List<AnnualPlan> {
        return annualPlanRepo.findAll()
    }

    fun evaluate(id: Long, evaluation: String): Boolean {
        val optional = get(id);
        if(optional.isPresent){
            val plan = optional.get()
            plan.evaluation = evaluation
            annualPlanRepo.save(plan)
            return true
        }
        return false
    }

}