package com.yaoxiang.planning.service

import com.yaoxiang.planning.domain.QuarterlyPlan
import com.yaoxiang.planning.model.PlanningType
import com.yaoxiang.planning.repository.QuarterlyPlanRepo
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import java.util.*

@Service
class QuarterlyPlanService {

    @Autowired
    private lateinit var quarterlyPlanRepo: QuarterlyPlanRepo

    @Autowired
    private lateinit var planningService: PlanningService

    @Autowired
    private lateinit var departmentService: DepartmentService


    fun add(name: String, content: String, year: Int, quarter: Int, departmentId: Long?): Boolean {
//        if (exists(year, quarter)) {
//            return false
//        }
        val planning = QuarterlyPlan()
        planning.quarter = quarter
        planning.type = PlanningType.QUARTERLY.ordinal
        planning.startTime = Date()
        planning.endTime = Date()
        planning.departmentId = departmentId
        planning.departmentName = departmentService.getName(departmentId)
        planningService.add(name, content, year, planning)
        return true
    }

    fun update(
        id: Long,
        name: String,
        content: String,
        year: Int,
        quarter: Int,
        status: Int,
        departmentId: Long?
    ): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        val planning = optional.get()
        planning.name = name
        planning.content = content
        planning.year = year
        planning.quarter = quarter
        planning.status = status
        planning.departmentId = departmentId
        planning.departmentName = departmentService.getName(departmentId)
        quarterlyPlanRepo.save(planning)
        return true
    }

    fun exists(year: Int, quarter: Int): Boolean {
        val optional = findByYearAndQuarter(year, quarter)
        return optional.isPresent
    }

    fun get(id: Long): Optional<QuarterlyPlan> {
        return quarterlyPlanRepo.findById(id)
    }

    fun findByYearAndQuarter(year: Int, quarter: Int): Optional<QuarterlyPlan> {
        return quarterlyPlanRepo.findFirstByYearAndQuarter(year, quarter)
    }

    fun delete(id: Long): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        quarterlyPlanRepo.delete(optional.get())
        return true
    }

    fun list(year: Int, departmentId: Long?): List<QuarterlyPlan> {
        if (departmentId == null) {
            return quarterlyPlanRepo.findByYear(year)
        }
        return quarterlyPlanRepo.findByYearAndDepartmentId(year, departmentId)
    }

    fun evaluate(id: Long, evaluation: String): Boolean {
        val optional = get(id)
        if (optional.isPresent) {
            val plan = optional.get()
            plan.evaluation = evaluation
            quarterlyPlanRepo.save(plan)
            return true
        }
        return false
    }


}