package com.yaoxiang.planning.service

import com.yaoxiang.planning.domain.WeeklyPlan
import com.yaoxiang.planning.model.PlanningType
import com.yaoxiang.planning.repository.WeeklyPlanRepo
import com.yaoxiang.planning.utils.DateUtil
import mu.KotlinLogging
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.util.*
import kotlin.collections.LinkedHashMap

private val logger = KotlinLogging.logger { }

@Service
class WeeklyPlanService {

    @Autowired
    private lateinit var weeklyPlanRepo: WeeklyPlanRepo

    @Autowired
    private lateinit var userService: UserService

    @Autowired
    private lateinit var planningService: PlanningService

    fun add(name: String, content: String, userId: Long?, startTime: Long, year: Int): Boolean {
        val planning = WeeklyPlan()
        planning.userId = userId ?: userService.currentUserId()
        planning.startTime = Date(startTime)
        planning.type = PlanningType.WEEKLY.ordinal
        planningService.add(name, content, year, planning)
        return true
    }

    fun update(id: Long, name: String, content: String, startTime: Long, year: Int, status: Int): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        val planning = optional.get()
        planning.name = name
        planning.content = content
        planning.startTime = Date(startTime)
        planning.year = year
        planning.status = status
        weeklyPlanRepo.save(planning)
        return true
    }

    fun get(id: Long): Optional<WeeklyPlan> {
        return weeklyPlanRepo.findById(id)
    }

    fun delete(id: Long): Boolean {
        val optional = get(id)
        if (!optional.isPresent) {
            return false
        }
        weeklyPlanRepo.delete(optional.get())
        return true
    }

    fun mapCurrentMonth(start: Long, userId: Long?, self: Boolean = true): Map<String, List<WeeklyPlan>> {
        logger.info { "mapCurrentMonth,${Date(start)}" }
        val list = findCurrentMoon(start, userId, self)
        val planList = mutableListOf<WeeklyPlan>()
        for (plan in list) {
            plan.dayOfMonth = DateUtil.getDay(plan.startTime)
            planList.add(plan)
        }
        val map = planList.groupBy { it.dayOfMonth }

        val monthDay = DateUtil.getMonthDay(Date(start))
        val result = LinkedHashMap<String, List<WeeklyPlan>>()
        for (i in 1..monthDay) {
            result[i.toString()] = map.getOrDefault(i, arrayListOf())
        }
        return result
    }

//    @Transactional(readOnly = true)
//    fun findCurrentWeek(start: Long, self: Boolean): List<WeeklyPlan> {
//        val time = Date(start)
//        val first = DateUtil.getFirstDayOfWeek(time)
//        val last = DateUtil.getLastDayOfWeek(time)
//        if (!self) {
//            return weeklyPlanRepo.findAllByStartTimeBetween(first, last)
//        }
//        val userId = userService.currentUserId()
//        return weeklyPlanRepo.findAllByUserIdAndStartTimeBetween(userId!!, first, last)
//    }

    @Transactional(readOnly = true)
    fun findCurrentMoon(start: Long, userId: Long?, self: Boolean = true): List<WeeklyPlan> {
        val id = userId ?: userService.currentUserId()
        val time = Date(start)
        val first = DateUtil.getFirstDayOfMonth(time)
        val last = DateUtil.getLastDayOfMonth(time)
//        logger.info { "first=${first}" }
//        logger.info { "last=${last}" }
        if (!self) {
            return weeklyPlanRepo.findAllByStartTimeBetween(first, last)
        }
        return weeklyPlanRepo.findAllByUserIdAndStartTimeBetween(id!!, first, last)
    }

    fun evaluate(id: Long, evaluation: String): Boolean {
        val optional = get(id)
        if (optional.isPresent) {
            val plan = optional.get()
            plan.evaluation = evaluation
            weeklyPlanRepo.save(plan)
            return true
        }
        return false
    }

}