package com.yaoxiang.planning.utils

import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap


/**
 * @date: 2020/8/26 12:30
 */
class TimeUtil {
    companion object {

        val formatter = SimpleDateFormat("yyyy-MM-dd HH:mm:ss")

        /**
         * 获取当天的凌晨0点0分0秒的时间
         */
        fun curDate(): Date {
            val calendar = Calendar.getInstance()
            calendar.time = Date()
            calendar[Calendar.HOUR_OF_DAY] = 0
            calendar[Calendar.MINUTE] = 0
            calendar[Calendar.SECOND] = 0
            return calendar.time
        }

        /**
         * 获取本周第一天的时间
         *
         * @return
         */
        fun curWeekFirstDate(): Date {
            val cal = Calendar.getInstance()
            cal.time = Date()
            // 获得当前日期是一个星期的第几天
            val dayWeek = cal[Calendar.DAY_OF_WEEK]
            if (1 == dayWeek) {
                cal.add(Calendar.DAY_OF_MONTH, -1)
            }
            // 设置一个星期的第一天，按中国的习惯一个星期的第一天是星期一
            cal.firstDayOfWeek = Calendar.MONDAY
            // 获得当前日期是一个星期的第几天
            val day = cal[Calendar.DAY_OF_WEEK]
            // 根据日历的规则，给当前日期减去星期几与一个星期第一天的差值
            cal.add(Calendar.DATE, cal.firstDayOfWeek - day)
            cal[Calendar.HOUR_OF_DAY] = 0
            cal[Calendar.MINUTE] = 0
            cal[Calendar.SECOND] = 0
            cal[Calendar.MILLISECOND] = 0
            return cal.time
        }

        /**
         * 本月第一天
         */
        fun curMonthFirstDate(): Date {
            val cal = Calendar.getInstance()
            cal.add(Calendar.MONTH, 0)
            cal[Calendar.DAY_OF_MONTH] = 1 //设置为1号,当前日期既为本月第一天
            cal[Calendar.HOUR_OF_DAY] = 0
            cal[Calendar.MINUTE] = 0
            cal[Calendar.SECOND] = 0
            cal[Calendar.MILLISECOND] = 0
            return cal.time
        }

        /**
         * 本年第一天
         */
        fun curYearFirstDate(): Date {
            val currCal = Calendar.getInstance()
            val calendar = Calendar.getInstance()
            calendar.clear()
            calendar[Calendar.YEAR] = currCal[Calendar.YEAR]
            return calendar.time
        }

        fun getStringDate(date: Date): String {
            return formatter.format(date)
        }

        /**
         * 字符串转时间date
         */
        fun getDateString(date: String): Date {
            return formatter.parse(date)
        }

        fun curYearFirstDateStirng(): String {
            return getStringDate(curYearFirstDate())
        }

        fun curMonthFirstDateString(): String {
            return getStringDate(curMonthFirstDate())
        }

        fun curWeekFirstDateString(): String {
            return getStringDate(curWeekFirstDate())
        }

        fun curDateString(): String {
            return getStringDate(curDate())
        }

        /**
         * 当前时间的字符串
         */
        fun curNowString(): String {
            return getStringDate(Date())
        }

        /**
         * 年月日时间字符 转为 年月日时分秒
         */
        fun dateToString(date: String): String {
            val parse = SimpleDateFormat("yyyy-MM-dd").parse(date)
            return formatter.format(parse)
        }

        /**
         * 判断一个指定时间，是否在一个时间段之内，精确到年月日
         */
        fun contains(time: String, beginTime: String, endTime: String): Boolean {
            val time = formatter.parse(time)
            val beginTime = formatter.parse(beginTime)
            val endTime = formatter.parse(endTime)
            return (time.time >= beginTime.time && time.time <= endTime.time)
        }

        /**
         * 比较时间大小（精确到时分秒）
         */
        fun compareDateString(time: String, otherTime: String): Int {
            val source = formatter.parse(time)
            val target = formatter.parse(otherTime)
            return source.compareTo(target)
        }


        /**
         * 获取之前数日的时间，清空时分秒
         */
        fun getNearlyDate(day: Int): Date {
            val now = Calendar.getInstance()
            now.add(Calendar.DAY_OF_MONTH, -day)
            now[Calendar.HOUR_OF_DAY] = 0
            now[Calendar.MINUTE] = 0
            now[Calendar.SECOND] = 0
            now[Calendar.MILLISECOND] = 0
            return now.time
        }

        fun getNearlyDateString(day: Int): String {
            return formatter.format(getNearlyDate(day))
        }

        /**
         * 获取最近30日的日期字符  list<map>
         */
        fun getNearlyDateStrings(): List<Map<String, Object?>> {
            val list = ArrayList<Map<String, Object?>>()
            for (index in 1..30) {
                val dateString = getNearlyDateString(index)
                val map = HashMap<String, Object?>()
                map.put(dateString, null)
                list.add(map)
            }
            return list
        }

        /**
         * 获取最近30日的日期字符  list
         */
        fun getNearlyDateStringList(): List<String> {
            val list = ArrayList<String>()
            for (index in 1..30) {
                val dateString = SimpleDateFormat("yyyy-MM-dd").format(getNearlyDate(index))
                list.add(dateString)
            }
            return list
        }

        /**
         * 是否同一天
         */
        fun isSameDate(time: String?, otherTime: String?): Boolean {
            val cal1 = Calendar.getInstance()
            cal1.time = formatter.parse(time)
            val cal2 = Calendar.getInstance()
            cal2.time = formatter.parse(otherTime)
            val isSameYear = cal1[Calendar.YEAR] == cal2[Calendar.YEAR]
            val isSameMonth = (isSameYear && cal1[Calendar.MONTH] == cal2[Calendar.MONTH])
            return (isSameMonth && cal1[Calendar.DAY_OF_MONTH] == cal2[Calendar.DAY_OF_MONTH])
        }

        /**
         * 是否同一天
         */
        fun isSameDateByDate(time: String?, otherTime: String?): Boolean {
            val cal1 = Calendar.getInstance()
            cal1.time = SimpleDateFormat("yyyy-MM-dd").parse(time)
            val cal2 = Calendar.getInstance()
            cal2.time = SimpleDateFormat("yyyy-MM-dd").parse(otherTime)
            val isSameYear = cal1[Calendar.YEAR] == cal2[Calendar.YEAR]
            val isSameMonth = (isSameYear && cal1[Calendar.MONTH] == cal2[Calendar.MONTH])
            return (isSameMonth && cal1[Calendar.DAY_OF_MONTH] == cal2[Calendar.DAY_OF_MONTH])
        }

        /**
         * 截取年月日部分时间串
         */
        fun getDateStringFromDate(date: String): String {
            return SimpleDateFormat("yyyy-MM-dd").format(formatter.parse(date))
        }

        /**
         * 获取两个时间差值
         */
        fun getDistanceTime(startTime: String, endTime: String): Long {
            var hour = 0
            val time1 = formatter.parse(startTime).time
            val time2 = formatter.parse(endTime).time
            val diff: Long
            diff = if (time1 < time2) {
                time2 - time1
            } else {
                time1 - time2
            }
//            hour = (diff / (60 * 60 * 1000)).toInt()
            return diff
        }


        /**
         * 距离指定时间的第二天凌晨0点还有多长时间
         */
        fun dayToTomorrowTime(startTime: String): Long {
            val date = formatter.parse(startTime)
            val calendar = Calendar.getInstance()
            calendar.time = date
            calendar.add(Calendar.DAY_OF_YEAR, 1)
            calendar.set(Calendar.HOUR_OF_DAY, 0)
            calendar.set(Calendar.MINUTE, 0)
            calendar.set(Calendar.SECOND, 0)
            calendar.set(Calendar.MILLISECOND, 0)
            return calendar.timeInMillis - date.time
        }

        /**
         * 指定时间的凌晨0:0:0
         */
        fun getDateZero(time: String): String {
            val calendar = Calendar.getInstance()
            calendar.time = formatter.parse(time)
            calendar[Calendar.HOUR_OF_DAY] = 0
            calendar[Calendar.MINUTE] = 0
            calendar[Calendar.SECOND] = 0
            return formatter.format(calendar.time)
        }
    }

}